/** \file main.c
 *
 * \brief UART: remote echo.
 *
 * This example program receives characters for the console and sends them back.
 *
 * The relevant registers are:
 * - CTRLB: USART Control Register B
 *   - RXEN: Receiver Enable
 *   - TXEN: Transmitter Enable
 * - CTRLC: USART Control Register C
 *   - PMODE: Parity Mode
 *   - SBMODE: Stop Bit Mode
 *   - CHSIZE: Character Size
 * - BAUD: USART Baud Rate Register
 *   - BAUD: Baud Rate
 * - STATUS: USART Status Register
 *   - RXCIF: Receive Complete Interrupt Flag
 *   - TXCIF: Transmit Complete Interrupt Flag
 * - RXDATAL: USART Receiver Data Register Low Byte
 *   - DATA: Receive Data
 * - TXDATAL: USART Transmit Data Register Low Byte
 *   - DATA: Transmit Data 
 *
 * Please see modules 4.2.5 and 4.3.5 for more information.
 *
 * \note
 * A console program sends the characters typed on the keyboard to another device and displays the
 * characters received from that device on the screen. We do not see anything when typing at the
 * console as the characters are directly sent to the remote device. Only characters which we
 * receive are displayed. So the remote device has to send the characters back in order to see
 * them. This is called remote echo.
 *
 * \par Baud Rate Calculation
 * In NORMAL mode the BAUD register value is calculated by the following formula:
 * BAUD = 4*F_CPU / BPS
 * * BAUD : BAUD register value
 * * F_CPU: System clock frequency
 * * BPS  : Bits per second (baud rate)
 *
 * \par Baud Rate Table (F_CPU = 3333333)
 * | bps    | BAUD |
 * |-------:|-----:|
 * |   9600 | 1389 |
 * |  14400 |  926 |
 * |  19200 |  694 |
 * |  38400 |  347 |
 * |  57600 |  231 |
 * | 115200 |  116 |
 * 
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-15
 *
 * \par History
 *      Created: 2018-07-15
 */

#include <avr/io.h>
#include "board.h"

int main(void)
{
	// We need a buffer to store the data received.
	volatile uint8_t buffer;

	/***** Configure the USART pins *****/
	// The TX pin is output and idle at first. Idle means that it must be set high.
	VCOM_PORT.OUT |= VCOM_TX_PIN_bm;
	VCOM_PORT.DIR |= VCOM_TX_PIN_bm;
	// The RX pin is input. We ensure this by clearing DIR although this is default.
	VCOM_PORT.DIR &= ~VCOM_RX_PIN_bm;

    /***** Configure the USART module *****/
	// We are using the virtual COM port. Its configuration is 115200 8N1.
    // Baud rate is 115200 bps. See baud rate table.
    VCOM_MODULE.BAUD = 116;
	// Line configuration is 8 data bits, no parity, 1 stop bit.
	VCOM_MODULE.CTRLC = (USART_CHSIZE_8BIT_gc | USART_PMODE_DISABLED_gc);
	// Enable receiver and transmitter.
	VCOM_MODULE.CTRLB = (USART_RXEN_bm | USART_TXEN_bm);

    // We use non-blocking receive and blocking send for the echo.
	while(1)
	{
		// Receive complete? If true the receive complete flag is set.
		if (VCOM_MODULE.STATUS & USART_RXCIF_bm) {
            // Read data and store it in the buffer.
			buffer = VCOM_MODULE.RXDATAL;
			// While the data register empty flag is not set do nothing.
			while(!(VCOM_MODULE.STATUS & USART_DREIF_bm));
			// Now the data register is empty and we can send the data safely.
			VCOM_MODULE.TXDATAL = buffer;
		}
	}
}
