/** \file main.c
 *
 * \brief PWM: dim the LED.
 *
 * This example program dims the LED with the aid of pulse-width modulation.
 *
 * The relevant registers are:
 * - TCAROUTEA: TCA Route Register A
 *   - TCA0: Alternative output pins for TCA0
 * - CTRLA: Control Register A
 *   - ENABLE: Enable clock 
 *   - CLKSEL: Clock divider select 
 * - CTRLB: Control Register B
 *   - HCMPnEN: High byte compare n enable
 *   - LCMPnEN: Low byte compare n enable
 * - CTRLD: Control Register D
 *   - SPLITM: Enable split mode 
 * - HCMPn: High byte compare or Capture Register
 * - LCMPn: Low byte compare or Capture Register
 * - HPER: High byte timer period register
 * - LPER: Low byte timer period register 
 *
 * The value for CMP is calculated by CMP = (PER+1)*d-1 where
 * - d: duty cycle
 * 
 * \Note
 * The wave output of TCA0 can be deviated to different ports. This
 * is done with the aid of PORTMUX. TCA0 can run in 16-bit normal
 * mode and in 8-bit split mode. In order to be able to use pins
 * 3, 4 and 5 for wave output split mode has to be used. In split
 * mode the counter high byte and counter low byte are independent
 * and single-slope PWM waveform generation mode is implicitely
 * seleted.
 * 
 * Please see modules 4.2.4 and 4.3.4 for more information.
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-15
 *
 * \par History
 *      Created: 2018-07-15
 */

#include <avr/io.h>

#include "board.h"
#include "switch.h"

// The LED is on PB5. We have to redirect the wave output of TCA0 to PORTB. 
// and we have to use split mode as pin 5 is driven by HCMP2. The LED is
// active low. This has to be taken into account when handling the dimming.
// We have two options to handle this issue:
// - We can invert the duty cycle: 0% => LED on, 25% => LED dimmed to 75% etc.
// - We can invert the pin: the active low LED becomes active high.
int main(void)
{
    int8_t duty = 100;

	// Initialize the LED and invert the pin.
    LED_PORT.DIR = LED_PIN_bm;
	LED_PINCTRL = PORT_INVEN_bm;

	// Initialize the push-button.
	BUTTON_PINCTRL = PORT_PULLUPEN_bm;

	/***** Initialize the PWM *****/
	// Redirect wave output pins to PORTB.
    PORTMUX.TCAROUTEA = PORTMUX_TCA0_PORTB_gc;
	// Enable split mode. This implies single-slope PWM mode.
	TCA0.SPLIT.CTRLD = TCA_SPLIT_SPLITM_bm;
	// Enable high byte compare register 2 in order to drive pin 5.
	TCA0.SPLIT.CTRLB = TCA_SPLIT_HCMP2EN_bm;
	// Set high byte period register. 
	TCA0.SPLIT.HPER = 100;
	// Set high byte compare register n.
	TCA0.SPLIT.HCMP2 = 100;
	// Enable timer and select prescaler.
	TCA0.SPLIT.CTRLA = TCA_SPLIT_CLKSEL_DIV1_gc | TCA_SPLIT_ENABLE_bm;

	while(1)
	{
		// Reduce duty cycle by 25% on each key press.
		if (SWITCH_PRESSED()) {
			duty -= 25;
			if (duty < 0) duty = 100;
			TCA0.SPLIT.HCMP2 = duty;
		}
	}
}
