/** \file main.c
 *
 * \brief ADC: read a sensor.
 *
 * This example program reads the internal temperature sensor and prints
 * the result of the ADC conversion at the console.
 *
 * The relevant registers are:
 * - COMMAND: ADC Control Register
 *   - STCONV: Start Conversion
 * - CTRLA: Control Register A
 *   - ENABLE: ADC Enable
 * - CTRLC: Control Register B
 *   - SAMPCAP: Sample Capacitance Selection
 *   - REFSEL: Reference Selection
 *   - PRESC: Prescaler 
 * - INTFLAGS: ADC Interrupt Flag Register
 *   - RESRDY: Result Ready Interrupt Flag
 * - MUXPOS: Analog Input Control Register
 * - RES: ADC Result Register
 *
 * The relevant register for the voltage reference is:
 * - VREF.CTRLA: Voltage Reference Control Register A  
 *   - ADC0REFSEL: ADC0 Reference Select
 *
 * The ADC conversion result is calculated by ADC = 1023*Vin/Vref.
 * The ADC clock frequency ranges from 100 kHz to 260 kHz if Vref = 0.55 V.
 * The ADC clock frequency ranges from 200 kHz to 1.5 MHz if Vref > 0.55 V.
 *
 * Please see modules 4.2.4 and 4.3.4 for more information.
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-17
 *
 * \par History
 *      Created: 2018-07-1
 */

#include <avr/io.h>
#include <util/delay.h>

#include "board.h"
#include "console.h"

#define SENSOR_MODULE ADC0
#define SENSOR_MUXPOS ADC_MUXPOS_AIN2_gc

// We use the ADC to read the internal temperature sensor. Chapter 28.3.2.6 of
// the datasheet explains how this is done. The sampling delay and the sample
// length are specific to the internal temperatur sensor and not relevant for
// this example. The reference voltage has to be 1.1 V. The size of the sampling
// capacitance should be reduced if the reference voltage is above 1 V. We 
// use a clock divider of 4 so that the ADC clock frequency is within the
// allowed range (CLK_ADC = 833 kHz). 
int main(void)
{
	volatile uint16_t temperature;

	console_init();

    /***** This section is specific to the internal temperatur sensor. *****/
	SENSOR_MODULE.SAMPCTRL = 27;
	SENSOR_MODULE.CTRLD = ADC_INITDLY_DLY32_gc;

	/***** Initialize ADC. *****/
	// Configure Voltage Reference.
	VREF.CTRLA = VREF_ADC0REFSEL_1V1_gc;
	// Select internal temperature sensor as analog input.
	SENSOR_MODULE.MUXPOS = ADC_MUXPOS_TEMPSENSE_gc;
    // Select reduced sampling capacitance and prescale by 4.
	SENSOR_MODULE.CTRLC = ADC_SAMPCAP_bm | ADC_PRESC_DIV4_gc;
	// Enable the ADC module.
	SENSOR_MODULE.CTRLA = ADC_ENABLE_bm;

	while(1)
	{
		// Start conversion.
		SENSOR_MODULE.COMMAND = ADC_STCONV_bm;
		// Wait until the conversion has finished.
		while(!(SENSOR_MODULE.INTFLAGS & ADC_RESRDY_bm));
		// Read the result of the conversion. 
		temperature = SENSOR_MODULE.RES;
		// Print the result.
		printf("%d\n", temperature);
		// Wait a bit until starting the next conversion.
		_delay_ms(1000);
	}
}
