/** \file main.c
 *
 * \brief SPI: data transfer.
 *
 * This example program transmits data via the Serial Peripheral Interface.
 *
 * The relevant registers are:
 * - CTRLA: SPI Control Register A
 *   - ENABLE: SPI Enable
 *   - MASTER: Master/Slave Select
 *   - PRESCALER: Clock Prescaler
 * - DATA: SPI Data Register
 * - INTFLAGS: SPI Interrupt Flags
 *   - IF: Receive Complete Interrupt Flag
 *
 * Please see module 5.3.3 for more information.
 *
 * This example uses the Data Gateway Interface to send data to the Data
 * Visualizer. In order to connect the board proceed as follows:
 *   - Open Data Visualizer via the tools menu.
 *   - Open the DGI Control Panel via Configuration > External Connection > DGI.
 *   - Open a Terminal window via  Configuration > Visualization > Terminal.
 *   - Connect the ATmega4809 Xplainded board.
 *   - Enable the SPI Interface.
 *   - Drag the SPI plug to the terminal socket.
 *   - Start the DGI session.
 * A sequences of As should be seen in the terminal windows. Please refer to
 * the Data Visualizer Software User's Guide for further details.
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-16
 *
 * \par History
 *      Created: 2018-07-16
 *
 */

#include <avr/io.h>
#include <util/delay.h>

#include "board.h"

// We will transmit data via the data gateway interface. The transmitted data
// can be received with the Data Visualizer tool. The microcontroller is the
// master of the transmission. MISO is input. MOSI, SS and SCK are output. We
// use a prescaler of 64. Therefore the transmission speed is 52 kbps. The
// slave select line is active low. The slave must be deselected at first.
int main(void)
{
	/***** Initialize the SPI. *****/
	// Deactivate the slave.
	DGI_SS_PORT.OUT |= DGI_SS_PIN_bm;
	DGI_SS_PORT.DIR |= DGI_SS_PIN_bm;
	// Configure the SPI lines.
	DGI_SPI_PORT.DIR &= ~DGI_MISO_PIN_bm;
	DGI_SPI_PORT.DIR |= DGI_MOSI_PIN_bm | DGI_SCK_PIN_bm;
	// Enable SPI, select master mode and set the prescaler factor to 64.
	DGI_SPI_MODULE.CTRLA = SPI_ENABLE_bm | SPI_MASTER_bm | SPI_PRESC_DIV64_gc;

    while(1)
    {
		// Select the slave.
		DGI_SS_PORT.OUT &= ~DGI_SS_PIN_bm;
		// Send the data.
		DGI_SPI_MODULE.DATA = 'A';
		// While transmission has not yet finished do nothing.
		while (!(DGI_SPI_MODULE.INTFLAGS & SPI_IF_bm));
		// Deselect the slave.
		DGI_SS_PORT.OUT |= DGI_SS_PIN_bm;
		// Wait a bit before starting the next transmission.
		_delay_ms(1000);
    }
}
