/** \file main.c
 *
 * \brief I2C: data transfer.
 *
 * This example program sends data via the Inter-Integrated Circuit Interface. The
 * slave has the address 0x20. Pull-up resistors have to be present on the bus.
 *
 * The relevant registers are:
 * - TWISPIROUTEA: Master Control Register C
 *   - = TWI_MASTER_ENABLE_bm;
 * - MADDR: TWI Master Target Address Register
 * - MBAUD: TWI Master Baud Rate Register
 * - MCTRLA: TWI Master Control Register A
 *   - ENABLE:?Enable TWI Master 
 * - MCTRLB: TWI Master Control Register B
 *   - MCMD: Command
 *     - TWI_MCMD_NOACT_gc: No Action
 *     - TWI_MCMD_RECVTRANS_gc: Receive or Transmit Data
 *     - TWI_MCMD_STOP_gc: Issue Stop Condition
 * - MDATA: TWI Master Data Register
 * - MSTATUS: TWI Master Status Register
 *   - WIF: Write Interrupt Flag
 *   - BUSSTATE: Bus State
 *
 * The Baud rate is calculated by BAUD = F_CPU/(2*F_I2C)-5 where
 * - F_CPU: System clock frequency
 * - F_I2C: I2C transmission speed
 * The maximum transmission rate is standard mode (100 kbit/s, F_I2C=100000).
 *
 * Please see module 5.3.3 for more information.
 *
 * This example uses the Data Gateway Interface to send data to the Data
 * Visualizer. In order to connect the board proceed as follows:
 *   - Open Data Visualizer via the tools menu.
 *   - Open the DGI Control Panel via Configuration > External Connection > DGI.
 *   - Open a Terminal window via  Configuration > Visualization > Terminal.
 *   - Connect the ATmega4809 Xplainded board.
 *   - Enable the TWI Interface.
 *   - Drag the TWI plug to the terminal socket.
 *   - Start the DGI session.
 * A sequences of As should be seen in the terminal windows. Please refer to
 * the Data Visualizer Software User's Guide for further details.
 *
 * \note
 * Atmel calls the bus Two-Wire Serial Interface (TWI). So the registers
 * and the bit names start with TWI.
 *
 * \par Bus Speeds (F_CPU = 3333333)
 *      | Mode      | Speed      | BAUD | Error |
 *      |-----------|------------|------|-------|
 *      | Low-speed |  10 kbit/s |  162 |   0%  |
 *      | Standard  | 100 kbit/s |   12 |  -2%  |
 *
 * \author    Wolfgang Neff
 * \version   1.0
 * \date      2018-07-16
 *
 * \par History
 *      Created: 2018-07-16
 *
 */

#include <avr/io.h>
#include <util/delay.h>

#include "board.h"



#define I2C_DATA 'A'
#define I2C_ADDRESS 0x28

// We will transmit data via the data gateway interface. The transmitted data
// can be received with the Data Visualizer tool. The microcontroller is the
// bus master of the transmission. We use standard mode (100 kbit/s) for
// the transmission speed.
int main(void)
{
	/***** Initialize the I2C module. *****/
	// Redirect I2C to PORTC.
	PORTMUX.TWISPIROUTEA = PORTMUX_TWI0_ALT2_gc;
	// Set baud rate.
	DGI_I2C_MODULE.MBAUD = 12;
	// Enable master mode.
	DGI_I2C_MODULE.MCTRLA = TWI_ENABLE_bm;
	// Force bus to be in idle state.
	DGI_I2C_MODULE.MSTATUS = TWI_BUSSTATE_IDLE_gc;
	
	while(1)
	{
	    /***** Open write transmission. *****/
		// Send start condition followed by the address with write indicator. 
		DGI_I2C_MODULE.MADDR = I2C_ADDRESS<<1;
		// Wait until the transmission has finished.
		while (!(DGI_I2C_MODULE.MSTATUS & TWI_WIF_bm));
		// No action by master necessary.
		DGI_I2C_MODULE.MCTRLB = TWI_MCMD_NOACT_gc;

	    /***** Send the data. *****/
		// Put the data onto the bus.
		DGI_I2C_MODULE.MDATA = I2C_DATA;
		// Wait until the transmission has finished.
		while (!(DGI_I2C_MODULE.MSTATUS & TWI_WIF_bm));
		// Finish the transmission.
		DGI_I2C_MODULE.MCTRLB = TWI_MCMD_RECVTRANS_gc;

	    /***** Close transmission *****/
		// Send stop condition.
		DGI_I2C_MODULE.MCTRLB = TWI_MCMD_STOP_gc;

		// Wait a bit before starting the next transmission.
		_delay_ms(1000);
	}
}
